%{
#include <stdio.h>
#include <math.h>
#include "calc.h"

#ifdef YYDEBUG
	extern int yydebug;
	yydebug=1;
#endif

bool noPrint=false;
double variable[26];

%}
%union {
	double	fval;
	int		ival;
}
%token <fval> REAL
%token <ival> INTEGER
%token <ival> VARIABLE
%token <fval> PI

%token SIN COS TAN ASIN ACOS ATAN CSC SEC COT
%token LOG LG LN EXP E
%token SQRT RINT CEIL FLOOR
%token MAX_X MIN_X SUM_X MEAN_X VAR_X
%token MAX_Y MIN_Y SUM_Y MEAN_Y VAR_Y
%token CLEAR ADD DEL SHOW EXIT
%left NEG
%right '='
%left ','
%left '-' '+'
%left '/' '*' '%'
%right '^'
%left 'C' 'P' '!'
%type <fval> expression
%%

program:
	|statement '\n'				{/*yyerrok;*/}
;
statement:
	| expression  				{if(noPrint==false) printf("=%g\n", $1);
								noPrint=false;}
	| VARIABLE '=' expression 	{ variable[$1] = $3;}
	| VARIABLE 					{printf("%g\n", variable[$1]);}

	| SHOW 						{show_list();}
	| CLEAR						{clear();}
	| DEL INTEGER 				{del($2);}

	| EXIT 						{status='q';}
;
expression:
	 expression ',' expression	{add($1,$3);noPrint=false;}
	| CEIL '(' expression ')'	{ $$ = ceil($3);}
	| RINT '(' expression ')'	{ $$ = rint($3);}
	| FLOOR '(' expression ')'	{ $$ = floor($3);}

	| SIN '(' expression ')'	{ $$ = sin($3); }
	| COS '(' expression ')'	{ $$ = cos($3); }
	| TAN '(' expression ')'	{ $$ = tan($3); }
	| SEC '(' expression ')'	{ $$ = 1/sin($3); }
	| CSC '(' expression ')'	{ $$ = 1/cos($3); }
	| COT '(' expression ')'	{ $$ = 1/tan($3); }
	| ASIN '(' expression ')'	{ $$ = asin($3); }
	| ACOS '(' expression ')'	{ $$ = acos($3); }
	| ATAN '(' expression ')'	{ $$ = atan($3); }

	| SQRT '(' expression ')'	{ $$ = sqrt ($3); }
	| expression '^' expression	{ $$ = pow( $1, $3);}
	| E '^' expression			{ $$ = exp($3);}
	| EXP '(' expression ')'	{ $$ = exp($3);}
	| LOG '(' expression ')'	{ $$ = log10($3);}
	| LN '(' expression ')'		{ $$ = log($3);}
	| LG '(' expression ')'		{ $$ = log($3)/log(2);}
	
	| expression '!'			{ $$ = factorial ($1); }
	| expression 'C' expression	{ $$ = combi($1, $3);}
	| expression 'P' expression	{ $$ = permu($1, $3);}

	| expression '+' expression	{ $$ = $1 + $3; }
	| expression '-' expression	{ $$ = $1 - $3; }
	| expression '*' expression	{ $$ = $1 * $3; }
	| expression '/' expression	{ $$ = $1 / $3; }
	| expression '%' expression	{ $$ = mod($1, $3);}
	
	| '-' expression %prec NEG	{ $$ = -$2; }
	| '(' expression ')'		{ $$ = $2; }

	| VARIABLE					{ $$ =variable[$1];}

	| MAX_X						{ $$ = max(list_X);}
	| MIN_X						{ $$ = min(list_X);}
	| SUM_X						{ $$ = sum(list_X,0);}
	| MEAN_X					{ $$ = mean(list_X);}
	| VAR_X						{ $$ = var(list_X);}

	| MAX_Y						{ $$ = max(list_Y);}
	| MIN_Y						{ $$ = min(list_Y);}
	| SUM_Y						{ $$ = sum(list_Y,0);}
	| MEAN_Y					{ $$ = mean(list_Y);}
	| VAR_Y						{ $$ = var(list_Y);}

	| REAL						{ $$ = $1; }
	| INTEGER					{ $$ = $1; }
	| PI						{ $$ = $1; }
	| E							{ $$ = M_E; }

;

%%
yyerror (char *s)
{
	printf ("%s\n", s);
}
