%{
#include <stdio.h>
#include <math.h>
#include "calc.h"

#ifdef YYDEBUG
	extern int yydebug;
	yydebug=1;
#endif

bool noPrint=false;
bool funcDefined=false;
bool isQuit=false;
double variable[26];

%}
%union {
	double	fval;
	int		ival;
}
%token <fval> REAL
%token <ival> INTEGER
%token <ival> VARIABLE
%token <fval> PI
%token <fval> ANS;

%token SIN COS TAN ASIN ACOS ATAN CSC SEC COT
%token LOG LG LN EXP E C P Y
%token SQRT RINT CEIL FLOOR
%token INTEGRAL TO PRIME
%token MAX_X MIN_X SUM_X MEAN_X VAR_X
%token MAX_Y MIN_Y SUM_Y MEAN_Y VAR_Y
%token HELP CLEAR ADD DEL SHOW EXIT
%left NEG
%right '='
%left ','
%left '-' '+'
%left '/' '*' '%'
%right '^'
%left C P '!' SQRT RINT CEIL FLOOR LOG LG LN EXP E 
%left SIN COS TAN ASIN ACOS ATAN CSC SEC COT

%type <fval> expression
%type <fval> number
%%

statement_list:
	|'\n'						{}
	|statement '\n'				{}
;
statement:
	 error						{noPrint=true;}
	| expression  				{ans= $1; if(noPrint==false) printf("=%g\n", $1);}
	| VARIABLE '=' expression 	{ variable[$1]= $3;}

	| FUNC expression 			{
									if(funcDefined==false){
										memcpy(funcBuf, buf, strlen(buf));
										printf("Լ  Ͽϴ.\n");
									}else  if(noPrint==false){
										printf("̹ Լ  Ǿ ֽϴ. clearϼ.\n");
									}
									//variable[$1] = $3;
									ans=$2;
									funcDefined=true;
								}
	| INTEGRAL number TO number	{	if (funcDefined==true){
										integral($2,$4);
										printf("integral=%g\n",ans);
									}else{
										yyerror("Լ   ʾҽϴ.");
									}
								}

	| PRIME number				{	if (funcDefined==true){
										prime($2);
										printf("prime:=%g\n", ans);
									}else{ 
										yyerror("Լ   ʾҽϴ.");
									}

								}
	| HELP						{help();}	
	| SHOW 						{show_list();}
	| CLEAR						{clear();}
	| DEL INTEGER 				{del($2);}

	| EXIT 						{isQuit=true;}
;
expression:
	 '(' expression ')'		{ $$ = $2; }
	| expression ',' expression	{add($1,$3);}
	| CEIL expression 			{ $$ = ceil($2);}
	| RINT expression 			{ $$ = rint($2);}
	| FLOOR expression 		{ $$ = floor($2);}

	| SIN expression 			{ $$ = sin($2); }
	| COS expression		 	{ $$ = cos($2); }
	| TAN expression		 	{ $$ = tan($2); }
	| SEC expression		 	{ $$ = 1/sin($2); }
	| CSC expression 			{ $$ = 1/cos($2); }
	| COT expression 			{ $$ = 1/tan($2); }
	| ASIN expression 			{ $$ = asin($2); }
	| ACOS expression 			{ $$ = acos($2); }
	| ATAN expression 			{ $$ = atan($2); }

	| SQRT expression 			{ $$ = sqrt ($2); }
	| expression '^' expression	{ $$ = pow( $1, $3);}
	| E '^' expression			{ $$ = exp($3);}
	| EXP expression 			{ $$ = exp($2);}
	| LOG expression 			{ $$ = log10($2);}
	| LN expression 			{ $$ = log($2);}
	| LG expression				{ $$ = log($2)/log(2);}
	
	| expression '!'			{ $$ = factorial ($1); }
	| expression C expression	{ $$ = combi($1, $3);}
	| expression P  expression	{ $$ = permu($1, $3);}

	| expression '+' expression	{ $$ = $1 + $3; }
	| expression '-' expression	{ $$ = $1 - $3; }
	| expression '*' expression	{ $$ = $1 * $3; }
	| expression '/' expression	{ $$ = $1 / $3; }
	| expression '%' expression	{ $$ = mod($1, $3);}
	
	| '-' expression %prec NEG	{ $$ = -$2; }

	| MAX_X						{ $$ = max(list_X);}
	| MIN_X						{ $$ = min(list_X);}
	| SUM_X						{ $$ = sum(list_X,0);}
	| MEAN_X					{ $$ = mean(list_X);}
	| VAR_X						{ $$ = var(list_X);}

	| MAX_Y						{ $$ = max(list_Y);}
	| MIN_Y						{ $$ = min(list_Y);}
	| SUM_Y						{ $$ = sum(list_Y,0);}
	| MEAN_Y					{ $$ = mean(list_Y);}
	| VAR_Y						{ $$ = var(list_Y);}

	| number					{ $$ = $1; }
	| VARIABLE					{ $$ =variable[$1]; }

;
number: INTEGER					{ $$ = $1;}
	| REAL						{ $$ = $1;}
	| PI						{ $$ = M_PI; }
	| E							{ $$ = M_E; }
	| ANS						{ $$ = ans;}
;


%%
yyerror (char *s)
{
	if (noPrint==false){
		printf ("%s\n", s);
	}else return 0;
}
